@@ -30,6 +30,8 @@
 #include "nautilus-file-utilities.h"
 #include "nautilus-search-provider.h"
 #include "nautilus-search-engine.h"
+#include "nautilus-search-engine-model.h"
+
 #include <eel/eel-glib-extensions.h>
 #include <gtk/gtk.h>
 #include <gio/gio.h>
@@ -52,6 +54,8 @@ struct NautilusSearchDirectoryDetails {
 	GList *monitor_list;
 	GList *callback_list;
 	GList *pending_callback_list;
+
+	NautilusDirectory *base_model;
 };
 
 typedef struct {
@@ -73,9 +77,18 @@ typedef struct {
 	GHashTable *non_ready_hash;
 } SearchCallback;
 
+enum {
+	PROP_0,
+	PROP_BASE_MODEL,
+	PROP_QUERY,
+	NUM_PROPERTIES
+};
+
 G_DEFINE_TYPE (NautilusSearchDirectory, nautilus_search_directory,
 	       NAUTILUS_TYPE_DIRECTORY);
 
+static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };
+
 static void search_engine_hits_added (NautilusSearchEngine *engine, GList *hits, NautilusSearchDirectory *search);
 static void search_engine_hits_subtracted (NautilusSearchEngine *engine, GList *hits, NautilusSearchDirectory *search);
 static void search_engine_finished (NautilusSearchEngine *engine, NautilusSearchDirectory *search);
@@ -136,11 +149,17 @@ start_or_stop_search_engine (NautilusSearchDirectory *search, gboolean adding)
 	    search->details->pending_callback_list) &&
 	    search->details->query &&
 	    !search->details->search_running) {
+		NautilusSearchEngineModel *model_provider;
+
 		/* We need to start the search engine */
 		search->details->search_running = TRUE;
 		search->details->search_finished = FALSE;
 		ensure_search_engine (search);
-		nautilus_search_provider_set_query (NAUTILUS_SEARCH_PROVIDER (search->details->engine), search->details->query);
+		nautilus_search_provider_set_query (NAUTILUS_SEARCH_PROVIDER (search->details->engine),
+						    search->details->query);
+
+		model_provider = nautilus_search_engine_get_model_provider (search->details->engine);
+		nautilus_search_engine_model_set_model (model_provider, search->details->base_model);
 
 		reset_file_list (search);
 
@@ -655,6 +674,58 @@ search_is_editable (NautilusDirectory *directory)
 	return FALSE;
 }
 
+static void
+search_set_property (GObject *object,
+		     guint property_id,
+		     const GValue *value,
+		     GParamSpec *pspec)
+{
+	NautilusSearchDirectory *search = NAUTILUS_SEARCH_DIRECTORY (object);
+
+	switch (property_id) {
+	case PROP_BASE_MODEL:
+		nautilus_search_directory_set_base_model (search, g_value_get_object (value));
+		break;
+	case PROP_QUERY:
+		nautilus_search_directory_set_query (search, g_value_get_object (value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+static void
+search_get_property (GObject *object,
+		     guint property_id,
+		     GValue *value,
+		     GParamSpec *pspec)
+{
+	NautilusSearchDirectory *search = NAUTILUS_SEARCH_DIRECTORY (object);
+
+	switch (property_id) {
+	case PROP_BASE_MODEL:
+		g_value_set_object (value, nautilus_search_directory_get_base_model (search));
+		break;
+	case PROP_QUERY:
+		g_value_take_object (value, nautilus_search_directory_get_query (search));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+static void
+clear_base_model (NautilusSearchDirectory *search)
+{
+	if (search->details->base_model != NULL) {
+		nautilus_directory_file_monitor_remove (search->details->base_model,
+							&search->details->base_model);
+		g_clear_object (&search->details->base_model);
+	}
+}
+
 static void
 search_dispose (GObject *object)
 {
@@ -662,7 +733,9 @@ search_dispose (GObject *object)
 	GList *list;
 
 	search = NAUTILUS_SEARCH_DIRECTORY (object);
-	
+
+	clear_base_model (search);
+
 	/* Remove search monitors */
 	if (search->details->monitor_list) {
 		for (list = search->details->monitor_list; list != NULL; list = list->next) {
@@ -713,10 +786,7 @@ search_finalize (GObject *object)
 	NautilusSearchDirectory *search;
 
 	search = NAUTILUS_SEARCH_DIRECTORY (object);
-
 	g_free (search->details->saved_search_uri);
-	
-	g_free (search->details);
 
 	G_OBJECT_CLASS (nautilus_search_directory_parent_class)->finalize (object);
 }
@@ -724,18 +794,20 @@ search_finalize (GObject *object)
 static void
 nautilus_search_directory_init (NautilusSearchDirectory *search)
 {
-	search->details = g_new0 (NautilusSearchDirectoryDetails, 1);
+	search->details = G_TYPE_INSTANCE_GET_PRIVATE (search, NAUTILUS_TYPE_SEARCH_DIRECTORY,
+						       NautilusSearchDirectoryDetails);
 }
 
 static void
 nautilus_search_directory_class_init (NautilusSearchDirectoryClass *class)
 {
-	NautilusDirectoryClass *directory_class;
+	NautilusDirectoryClass *directory_class = NAUTILUS_DIRECTORY_CLASS (class);
+	GObjectClass *oclass = G_OBJECT_CLASS (class);
 
-	G_OBJECT_CLASS (class)->dispose = search_dispose;
-	G_OBJECT_CLASS (class)->finalize = search_finalize;
-
-	directory_class = NAUTILUS_DIRECTORY_CLASS (class);
+	oclass->dispose = search_dispose;
+	oclass->finalize = search_finalize;
+	oclass->get_property = search_get_property;
+	oclass->set_property = search_set_property;
 
  	directory_class->are_all_files_seen = search_are_all_files_seen;
 	directory_class->contains_file = search_contains_file;
@@ -748,6 +820,48 @@ nautilus_search_directory_class_init (NautilusSearchDirectoryClass *class)
 	
 	directory_class->get_file_list = search_get_file_list;
 	directory_class->is_editable = search_is_editable;
+
+	properties[PROP_BASE_MODEL] =
+		g_param_spec_object ("base-model",
+				     "The base model",
+				     "The base directory model for this directory",
+				     NAUTILUS_TYPE_DIRECTORY,
+				     G_PARAM_READWRITE);
+	properties[PROP_QUERY] =
+		g_param_spec_object ("query",
+				     "The query",
+				     "The query for this search directory",
+				     NAUTILUS_TYPE_QUERY,
+				     G_PARAM_READWRITE);
+
+	g_object_class_install_properties (oclass, NUM_PROPERTIES, properties);
+	g_type_class_add_private (class, sizeof (NautilusSearchDirectoryDetails));
+}
+
+void
+nautilus_search_directory_set_base_model (NautilusSearchDirectory *search,
+					  NautilusDirectory *base_model)
+{
+	if (search->details->base_model == base_model) {
+		return;
+	}
+
+	clear_base_model (search);
+	search->details->base_model = nautilus_directory_ref (base_model);
+
+	if (search->details->base_model != NULL) {
+		nautilus_directory_file_monitor_add (base_model, &search->details->base_model,
+						     TRUE, NAUTILUS_FILE_ATTRIBUTE_INFO,
+						     NULL, NULL);
+	}
+
+	g_object_notify_by_pspec (G_OBJECT (search), properties[PROP_BASE_MODEL]);
+}
+
+NautilusDirectory *
+nautilus_search_directory_get_base_model (NautilusSearchDirectory *search)
+{
+	return search->details->base_model;
 }
 
 char *
@@ -761,32 +875,29 @@ nautilus_search_directory_generate_new_uri (void)
 	return uri;
 }
 
-
 void
 nautilus_search_directory_set_query (NautilusSearchDirectory *search,
 				     NautilusQuery *query)
 {
-	NautilusDirectory *dir;
-	NautilusFile *as_file;
+	NautilusFile *file;
 
 	if (search->details->query != query) {
 		search->details->modified = TRUE;
+
+		if (query) {
+			g_object_ref (query);
+		}
+
+		g_clear_object (&search->details->query);
+		search->details->query = query;
+
+		g_object_notify_by_pspec (G_OBJECT (search), properties[PROP_QUERY]);
 	}
 
-	if (query) {
-		g_object_ref (query);
-	}
-
-	if (search->details->query) {
-		g_object_unref (search->details->query);
-	}
-
-	search->details->query = query;
-
-	dir = NAUTILUS_DIRECTORY (search);
-	as_file = dir->details->as_file;
-	if (as_file != NULL) {
-		nautilus_search_directory_file_update_display_name (NAUTILUS_SEARCH_DIRECTORY_FILE (as_file));
+	file = nautilus_directory_get_existing_corresponding_file (NAUTILUS_DIRECTORY (search));
+	if (file != NULL) {
+		nautilus_search_directory_file_update_display_name (NAUTILUS_SEARCH_DIRECTORY_FILE (file));
+		g_object_unref (file);
 	}
 }
 
