@@ -27,6 +27,7 @@
 #include "nautilus-search-provider.h"
 #include "nautilus-search-engine.h"
 #include "nautilus-search-engine-simple.h"
+#include "nautilus-search-engine-model.h"
 #define DEBUG_FLAG NAUTILUS_DEBUG_SEARCH
 #include "nautilus-debug.h"
 
@@ -36,12 +37,14 @@
 
 struct NautilusSearchEngineDetails
 {
-	NautilusSearchEngineSimple *simple;
 #ifdef ENABLE_TRACKER
 	NautilusSearchEngineTracker *tracker;
 #endif
+	NautilusSearchEngineSimple *simple;
+	NautilusSearchEngineModel *model;
+
 	GHashTable *uris;
-	guint num_providers;
+	guint providers_running;
 	guint providers_finished;
 	guint providers_error;
 };
@@ -62,6 +65,7 @@ nautilus_search_engine_set_query (NautilusSearchProvider *provider,
 #ifdef ENABLE_TRACKER
 	nautilus_search_provider_set_query (NAUTILUS_SEARCH_PROVIDER (engine->details->tracker), query);
 #endif
+	nautilus_search_provider_set_query (NAUTILUS_SEARCH_PROVIDER (engine->details->model), query);
 	nautilus_search_provider_set_query (NAUTILUS_SEARCH_PROVIDER (engine->details->simple), query);
 }
 
@@ -69,12 +73,22 @@ static void
 nautilus_search_engine_start (NautilusSearchProvider *provider)
 {
 	NautilusSearchEngine *engine = NAUTILUS_SEARCH_ENGINE (provider);
+
+	engine->details->providers_running = 0;
 	engine->details->providers_finished = 0;
 	engine->details->providers_error = 0;
+
 #ifdef ENABLE_TRACKER
 	nautilus_search_provider_start (NAUTILUS_SEARCH_PROVIDER (engine->details->tracker));
+	engine->details->providers_running++;
 #endif
-	nautilus_search_provider_start (NAUTILUS_SEARCH_PROVIDER (engine->details->simple));
+	if (nautilus_search_engine_model_get_model (engine->details->model)) {
+		nautilus_search_provider_start (NAUTILUS_SEARCH_PROVIDER (engine->details->model));
+		engine->details->providers_running++;
+	} else {
+		nautilus_search_provider_start (NAUTILUS_SEARCH_PROVIDER (engine->details->simple));
+		engine->details->providers_running++;
+	}
 }
 
 static void
@@ -84,6 +98,7 @@ nautilus_search_engine_stop (NautilusSearchProvider *provider)
 #ifdef ENABLE_TRACKER
 	nautilus_search_provider_stop (NAUTILUS_SEARCH_PROVIDER (engine->details->tracker));
 #endif
+	nautilus_search_provider_stop (NAUTILUS_SEARCH_PROVIDER (engine->details->model));
 	nautilus_search_provider_stop (NAUTILUS_SEARCH_PROVIDER (engine->details->simple));
 }
 
@@ -104,7 +119,7 @@ search_provider_hits_added (NautilusSearchProvider *provider,
 		count = GPOINTER_TO_INT (g_hash_table_lookup (engine->details->uris, uri));
 		if (count == 0)
 			added = g_list_prepend (added, hit);
-		g_hash_table_replace (engine->details->uris, g_strdup (uri), GINT_TO_POINTER (count++));
+		g_hash_table_replace (engine->details->uris, g_strdup (uri), GINT_TO_POINTER (++count));
 	}
 	if (added != NULL) {
 		added = g_list_reverse (added);
@@ -133,7 +148,7 @@ search_provider_hits_subtracted (NautilusSearchProvider *provider,
 			removed = g_list_prepend (removed, hit);
 			g_hash_table_remove (engine->details->uris, uri);
 		} else {
-			g_hash_table_replace (engine->details->uris, g_strdup (uri), GINT_TO_POINTER (count--));
+			g_hash_table_replace (engine->details->uris, g_strdup (uri), GINT_TO_POINTER (--count));
 		}
 	}
 	if (removed != NULL) {
@@ -150,9 +165,12 @@ search_provider_error (NautilusSearchProvider *provider,
 {
 	DEBUG ("Search provider error: %s", error_message);
 	engine->details->providers_error++;
-	if (engine->details->providers_error == engine->details->num_providers) {
+	if (engine->details->providers_error == engine->details->providers_running) {
+		g_object_ref (engine);
 		nautilus_search_provider_error (NAUTILUS_SEARCH_PROVIDER (engine),
 						_("Unable to complete the requested search"));
+		g_hash_table_remove_all (engine->details->uris);
+		g_object_unref (engine);
 	}
 }
 
@@ -162,8 +180,12 @@ search_provider_finished (NautilusSearchProvider *provider,
 
 {
 	engine->details->providers_finished++;
-	if (engine->details->providers_finished == engine->details->num_providers)
+	if (engine->details->providers_finished == engine->details->providers_running) {
+		g_object_ref (engine);
 		nautilus_search_provider_finished (NAUTILUS_SEARCH_PROVIDER (engine));
+		g_hash_table_remove_all (engine->details->uris);
+		g_object_unref (engine);
+	}
 }
 
 static void
@@ -202,6 +224,7 @@ nautilus_search_engine_finalize (GObject *object)
 #ifdef ENABLE_TRACKER
 	g_clear_object (&engine->details->tracker);
 #endif
+	g_clear_object (&engine->details->model);
 	g_clear_object (&engine->details->simple);
 
 	G_OBJECT_CLASS (nautilus_search_engine_parent_class)->finalize (object);
@@ -231,12 +254,12 @@ nautilus_search_engine_init (NautilusSearchEngine *engine)
 #ifdef ENABLE_TRACKER
 	engine->details->tracker = nautilus_search_engine_tracker_new ();
 	connect_provider_signals (engine, NAUTILUS_SEARCH_PROVIDER (engine->details->tracker));
-	engine->details->num_providers++;
 #endif
+	engine->details->model = nautilus_search_engine_model_new ();
+	connect_provider_signals (engine, NAUTILUS_SEARCH_PROVIDER (engine->details->model));
 
 	engine->details->simple = nautilus_search_engine_simple_new ();
 	connect_provider_signals (engine, NAUTILUS_SEARCH_PROVIDER (engine->details->simple));
-	engine->details->num_providers++;
 }
 
 NautilusSearchEngine *
@@ -248,3 +271,9 @@ nautilus_search_engine_new (void)
 
 	return engine;
 }
+
+NautilusSearchEngineModel *
+nautilus_search_engine_get_model_provider (NautilusSearchEngine *engine)
+{
+	return engine->details->model;
+}
