@@ -52,8 +52,6 @@
 #include "nautilus-window.h"
 #include "nautilus-window-slot.h"
 
-#include <libnotify/notify.h>
-
 #define DEBUG_FLAG NAUTILUS_DEBUG_PLACES
 #include <libnautilus-private/nautilus-debug.h>
 
@@ -100,8 +98,6 @@ typedef struct {
 	NautilusWindowSlot *go_to_after_mount_slot;
 	NautilusWindowOpenFlags go_to_after_mount_flags;
 
-	NotifyNotification *unmount_notify;
-
 	GDBusProxy *hostnamed_proxy;
 	char *hostname;
 
@@ -404,9 +400,9 @@ static GIcon *
 special_directory_get_gicon (GUserDirectory directory)
 {
 
-	#define ICON_CASE(x) \
-		case G_USER_DIRECTORY_ ## x:\
-			return g_themed_icon_new (NAUTILUS_ICON_FOLDER_ ## x);
+#define ICON_CASE(x)				\
+	case G_USER_DIRECTORY_ ## x:					\
+		return g_themed_icon_new (NAUTILUS_ICON_FOLDER_ ## x);
 
 	switch (directory) {
 
@@ -422,7 +418,7 @@ special_directory_get_gicon (GUserDirectory directory)
 		return g_themed_icon_new ("folder-symbolic");
 	}
 
-	#undef ICON_CASE
+#undef ICON_CASE
 }
 
 static gboolean
@@ -444,6 +440,62 @@ recent_is_supported (void)
 	return FALSE;
 }
 
+static void
+add_special_dirs (NautilusPlacesSidebar *sidebar)
+{
+	GList *dirs;
+	int index;
+
+	dirs = NULL;
+	for (index = 0; index < G_USER_N_DIRECTORIES; index++) {
+		const char *path;
+		GFile *root;
+		GIcon *icon;
+		char *name;
+		char *mount_uri;
+		char *tooltip;
+
+		if (index == G_USER_DIRECTORY_DESKTOP ||
+		    index == G_USER_DIRECTORY_TEMPLATES ||
+		    index == G_USER_DIRECTORY_PUBLIC_SHARE) {
+			continue;
+		}
+
+		path = g_get_user_special_dir (index);
+
+		/* xdg resets special dirs to the home directory in case
+		 * it's not finiding what it expects. We don't want the home
+		 * to be added multiple times in that weird configuration.
+		 */
+		if (path == NULL
+		    || g_strcmp0 (path, g_get_home_dir ()) == 0
+		    || g_list_find_custom (dirs, path, (GCompareFunc) g_strcmp0) != NULL) {
+			continue;
+		}
+
+		root = g_file_new_for_path (path);
+		name = g_file_get_basename (root);
+		icon = special_directory_get_gicon (index);
+		mount_uri = g_file_get_uri (root);
+		tooltip = g_file_get_parse_name (root);
+
+		add_place (sidebar, PLACES_XDG_DIR,
+			   SECTION_COMPUTER,
+			   name, icon, mount_uri,
+			   NULL, NULL, NULL, 0,
+			   tooltip);
+		g_free (name);
+		g_object_unref (root);
+		g_object_unref (icon);
+		g_free (mount_uri);
+		g_free (tooltip);
+
+		dirs = g_list_prepend (dirs, (char *)path);
+	}
+
+	g_list_free (dirs);
+}
+
 static void
 update_places (NautilusPlacesSidebar *sidebar)
 {
@@ -460,7 +512,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 	GVolume *volume;
 	int bookmark_count, index;
 	char *location, *mount_uri, *name, *last_uri, *identifier;
-	const gchar *path, *bookmark_name;
+	const gchar *bookmark_name;
 	GIcon *icon;
 	GFile *root;
 	NautilusWindowSlot *slot;
@@ -531,41 +583,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 	}
 
 	/* XDG directories */
-	for (index = 0; index < G_USER_N_DIRECTORIES; index++) {
-
-		if (index == G_USER_DIRECTORY_DESKTOP ||
-		    index == G_USER_DIRECTORY_TEMPLATES ||
-		    index == G_USER_DIRECTORY_PUBLIC_SHARE) {
-			continue;
-		}
-
-		path = g_get_user_special_dir (index);
-
-		/* xdg resets special dirs to the home directory in case
-		 * it's not finiding what it expects. We don't want the home
-		 * to be added multiple times in that weird configuration.
-		 */
-		if (!path || g_strcmp0 (path, g_get_home_dir ()) == 0) {
-			continue;
-		}
-
-		root = g_file_new_for_path (path);
-		name = g_file_get_basename (root);
-		icon = special_directory_get_gicon (index);
-		mount_uri = g_file_get_uri (root);
-		tooltip = g_file_get_parse_name (root);
-
-		add_place (sidebar, PLACES_XDG_DIR,
-			   SECTION_COMPUTER,
-			   name, icon, mount_uri,
-			   NULL, NULL, NULL, 0,
-			   tooltip);
-		g_free (name);
-		g_object_unref (root);
-		g_object_unref (icon);
-		g_free (mount_uri);
-		g_free (tooltip);
-	}
+	add_special_dirs (sidebar);
 
 	mount_uri = "trash:///"; /* No need to strdup */
 	icon = nautilus_trash_monitor_get_icon ();
@@ -747,6 +765,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 
 		if (!g_file_is_native (root)) {
 			network_mounts = g_list_prepend (network_mounts, mount);
+			g_object_unref (root);
 			continue;
 		}
 
@@ -788,7 +807,7 @@ update_places (NautilusPlacesSidebar *sidebar)
 		nautilus_file_unref (file);
 
 		bookmark_name = nautilus_bookmark_get_name (bookmark);
-		icon = nautilus_bookmark_get_icon (bookmark);
+		icon = nautilus_bookmark_get_symbolic_icon (bookmark);
 		mount_uri = nautilus_bookmark_get_uri (bookmark);
 		tooltip = g_file_get_parse_name (root);
 
@@ -1882,7 +1901,7 @@ add_bookmark (NautilusPlacesSidebar *sidebar)
 		}
 
 		location = g_file_new_for_uri (uri);
-		bookmark = nautilus_bookmark_new (location, name, NULL);
+		bookmark = nautilus_bookmark_new (location, name);
 
 		if (!nautilus_bookmark_list_contains (sidebar->bookmarks, bookmark)) {
 			nautilus_bookmark_list_append (sidebar->bookmarks, bookmark);
@@ -2767,6 +2786,7 @@ bookmarks_button_release_event_cb (GtkWidget *widget,
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	GtkTreeView *tree_view;
+	gboolean ret = FALSE;
 	gboolean res;
 
 	path = NULL;
@@ -2785,68 +2805,25 @@ bookmarks_button_release_event_cb (GtkWidget *widget,
 	tree_view = GTK_TREE_VIEW (widget);
 	model = gtk_tree_view_get_model (tree_view);
 
-	if (event->button == 1) {
-
-		if (event->window != gtk_tree_view_get_bin_window (tree_view)) {
-			return FALSE;
-		}
-
-		res = gtk_tree_view_get_path_at_pos (tree_view, (int) event->x, (int) event->y,
-						     &path, NULL, NULL, NULL);
-
-		if (!res) {
-			return FALSE;
-		}
-
-		gtk_tree_model_get_iter (model, &iter, path);
-
-		open_selected_bookmark (sidebar, model, &iter, 0);
-
-		gtk_tree_path_free (path);
-	}
-
-	return FALSE;
-}
-
-/* Callback used when a button is pressed on the shortcuts list.  
- * We trap button 3 to bring up a popup menu, and button 2 to
- * open in a new tab.
- */
-static gboolean
-bookmarks_button_press_event_cb (GtkWidget             *widget,
-				 GdkEventButton        *event,
-				 NautilusPlacesSidebar *sidebar)
-
-{
-	GtkTreeModel *model;
-	GtkTreeView *tree_view;
-	GtkTreeIter iter;
-	GtkTreePath *path = NULL;
-	gboolean retval = FALSE;
-	PlaceType row_type;
-
-	if (event->type != GDK_BUTTON_PRESS) {
-		/* ignore multiple clicks */
-		return TRUE;
-	}
-
-	tree_view = GTK_TREE_VIEW (widget);
-	model = gtk_tree_view_get_model (tree_view);
-	gtk_tree_view_get_path_at_pos (tree_view, (int) event->x, (int) event->y, 
-				       &path, NULL, NULL, NULL);
-
-	if (path == NULL || !gtk_tree_model_get_iter (model, &iter, path)) {
+	if (event->window != gtk_tree_view_get_bin_window (tree_view)) {
 		return FALSE;
 	}
 
-	if (event->button == 3) {
-		gtk_tree_model_get (model, &iter,
-				    PLACES_SIDEBAR_COLUMN_ROW_TYPE, &row_type,
-				    -1);
+	res = gtk_tree_view_get_path_at_pos (tree_view, (int) event->x, (int) event->y,
+					     &path, NULL, NULL, NULL);
 
-		if (row_type != PLACES_HEADING) {
-			bookmarks_popup_menu (sidebar, event);
-		}
+	if (!res || path == NULL) {
+		return FALSE;
+	}
+
+	res = gtk_tree_model_get_iter (model, &iter, path);
+	if (!res) {
+		gtk_tree_path_free (path);
+		return FALSE;
+	}
+
+	if (event->button == 1) {
+		open_selected_bookmark (sidebar, model, &iter, 0);
 	} else if (event->button == 2) {
 		NautilusWindowOpenFlags flags = 0;
 
@@ -2855,15 +2832,24 @@ bookmarks_button_press_event_cb (GtkWidget             *widget,
 			NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
 
 		open_selected_bookmark (sidebar, model, &iter, flags);
-		retval = TRUE;
+		ret = TRUE;
+	} else if (event->button == 3) {
+		PlaceType row_type;
+
+		gtk_tree_model_get (model, &iter,
+				    PLACES_SIDEBAR_COLUMN_ROW_TYPE, &row_type,
+				    -1);
+
+		if (row_type != PLACES_HEADING) {
+			bookmarks_popup_menu (sidebar, event);
+		}
 	}
 
 	gtk_tree_path_free (path);
 
-	return retval;
+	return ret;
 }
 
-
 static void
 bookmarks_edited (GtkCellRenderer       *cell,
 		  gchar                 *path_string,
@@ -3276,8 +3262,6 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 			  G_CALLBACK (bookmarks_selection_changed_cb), sidebar);
 	g_signal_connect (tree_view, "popup-menu",
 			  G_CALLBACK (bookmarks_popup_menu_cb), sidebar);
-	g_signal_connect (tree_view, "button-press-event",
-			  G_CALLBACK (bookmarks_button_press_event_cb), sidebar);
 	g_signal_connect (tree_view, "button-release-event",
 			  G_CALLBACK (bookmarks_button_release_event_cb), sidebar);
 
@@ -3319,7 +3303,6 @@ nautilus_places_sidebar_dispose (GObject *object)
 	sidebar->uri = NULL;
 
 	free_drag_data (sidebar);
-	g_clear_object (&sidebar->unmount_notify);
 
 	if (sidebar->bookmarks_changed_id != 0) {
 		g_signal_handler_disconnect (sidebar->bookmarks,
