@@ -31,7 +31,6 @@
 #include "nautilus-actions.h"
 #include "nautilus-application.h"
 #include "nautilus-floating-bar.h"
-#include "nautilus-location-bar.h"
 #include "nautilus-pathbar.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window-slot.h"
@@ -135,7 +134,7 @@ set_displayed_file (NautilusWindowSlot *slot, NautilusFile *file)
 		if (new_location == NULL) {
 			slot->current_location_bookmark = NULL;
 		} else {
-			slot->current_location_bookmark = nautilus_bookmark_new (new_location, display_name, NULL);
+			slot->current_location_bookmark = nautilus_bookmark_new (new_location, display_name);
 		}
 		g_free (display_name);
         }
@@ -972,6 +971,7 @@ create_content_view (NautilusWindowSlot *slot,
 	GList *selection;
 	gboolean ret = TRUE;
 	GError *error = NULL;
+	NautilusDirectory *old_directory, *new_directory;
 
 	window = nautilus_window_slot_get_window (slot);
 
@@ -1004,6 +1004,21 @@ create_content_view (NautilusWindowSlot *slot,
 		nautilus_window_connect_content_view (window, slot->new_content_view);
         }
 
+	/* Forward search selection and state before loading the new model */
+	new_directory = nautilus_directory_get (slot->pending_location);
+	old_directory = nautilus_directory_get (slot->location);
+
+	if (NAUTILUS_IS_SEARCH_DIRECTORY (new_directory) &&
+	    !NAUTILUS_IS_SEARCH_DIRECTORY (old_directory)) {
+		nautilus_search_directory_set_base_model (NAUTILUS_SEARCH_DIRECTORY (new_directory), old_directory);
+	}
+
+	if (NAUTILUS_IS_SEARCH_DIRECTORY (old_directory) &&
+	    !NAUTILUS_IS_SEARCH_DIRECTORY (new_directory) &&
+	    slot->pending_selection == NULL) {
+		slot->pending_selection = nautilus_view_get_selection (slot->content_view);
+	}
+
 	/* Actually load the pending location and selection: */
 
         if (slot->pending_location != NULL) {
@@ -1811,7 +1826,7 @@ nautilus_window_back_or_forward (NautilusWindow *window,
 
 /* reload the contents of the window */
 void
-nautilus_window_slot_reload (NautilusWindowSlot *slot)
+nautilus_window_slot_force_reload (NautilusWindowSlot *slot)
 {
 	GFile *location;
         char *current_pos;
@@ -1841,3 +1856,23 @@ nautilus_window_slot_reload (NautilusWindowSlot *slot)
 	g_object_unref (location);
 	g_list_free_full (selection, g_object_unref);
 }
+
+void
+nautilus_window_slot_queue_reload (NautilusWindowSlot *slot)
+{
+	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
+
+	if (slot->location == NULL) {
+		return;
+	}
+
+	if (slot->pending_location != NULL
+	    || slot->content_view == NULL
+	    || nautilus_view_get_loading (slot->content_view)) {
+		/* there is a reload in flight */
+		slot->needs_reload = TRUE;
+		return;
+	}
+
+	nautilus_window_slot_force_reload (slot);
+}
