@@ -98,7 +98,7 @@ sync_search_directory (NautilusWindowSlot *slot)
 	} else {
 		nautilus_search_directory_set_query (NAUTILUS_SEARCH_DIRECTORY (directory),
 						     query);
-		nautilus_window_slot_reload (slot);
+		nautilus_window_slot_force_reload (slot);
 	}
 
 	g_free (text);
@@ -106,24 +106,13 @@ sync_search_directory (NautilusWindowSlot *slot)
 	nautilus_directory_unref (directory);
 }
 
-static gboolean
-sync_search_location_cb (NautilusWindow *window,
-			 GError *error,
-			 gpointer user_data)
-{
-	NautilusWindowSlot *slot = user_data;
-	if (error == NULL) {
-		sync_search_directory (slot);
-	}
-	return (error == NULL);
-}
-
 static void
 create_new_search (NautilusWindowSlot *slot)
 {
 	char *uri;
 	NautilusDirectory *directory;
 	GFile *location;
+	NautilusQuery *query;
 
 	uri = nautilus_search_directory_generate_new_uri ();
 	location = g_file_new_for_uri (uri);
@@ -131,9 +120,13 @@ create_new_search (NautilusWindowSlot *slot)
 	directory = nautilus_directory_get (location);
 	g_assert (NAUTILUS_IS_SEARCH_DIRECTORY (directory));
 
-	nautilus_window_slot_open_location_full (slot, location, 0, NULL, sync_search_location_cb, slot);
+	query = nautilus_query_editor_get_query (slot->query_editor);
+	nautilus_search_directory_set_query (NAUTILUS_SEARCH_DIRECTORY (directory), query);
+
+	nautilus_window_slot_open_location (slot, location, 0);
 
 	nautilus_directory_unref (directory);
+	g_object_unref (query);
 	g_object_unref (location);
 	g_free (uri);
 }
@@ -379,6 +372,17 @@ nautilus_window_slot_init (NautilusWindowSlot *slot)
 		(slot, NAUTILUS_TYPE_WINDOW_SLOT, NautilusWindowSlotDetails);
 }
 
+static void
+view_end_loading_cb (NautilusView       *view,
+		     gboolean            all_files_seen,
+		     NautilusWindowSlot *slot)
+{
+	if (slot->needs_reload) {
+		nautilus_window_slot_queue_reload (slot);
+		slot->needs_reload = FALSE;
+	}
+}
+
 static void
 nautilus_window_slot_dispose (GObject *object)
 {
@@ -588,6 +592,8 @@ nautilus_window_slot_set_content_view_widget (NautilusWindowSlot *slot,
 
 	if (slot->content_view != NULL) {
 		/* disconnect old view */
+		g_signal_handlers_disconnect_by_func (slot->content_view, G_CALLBACK (view_end_loading_cb), slot);
+
 		nautilus_window_disconnect_content_view (window, slot->content_view);
 
 		widget = GTK_WIDGET (slot->content_view);
@@ -604,6 +610,8 @@ nautilus_window_slot_set_content_view_widget (NautilusWindowSlot *slot,
 		slot->content_view = new_view;
 		g_object_ref (slot->content_view);
 
+		g_signal_connect (new_view, "end_loading", G_CALLBACK (view_end_loading_cb), slot);
+
 		/* connect new view */
 		nautilus_window_connect_content_view (window, new_view);
 	}
